TITAN_DEFENSE_FREQUENCY = 1;
TITAN_DEFENSE_ID = "Defense";

DEFENSETRACKER_STRINGS_ZONEATTACKSEARCH = string.gsub(ZONE_UNDER_ATTACK, "%%[^%s]+", "(.+)") -- "|cffffff00%s is under attack!|r"

-- Options
TitanDefense_warningDuration = 40;

-- Variables for important info
DefenseTracker_uploadNotRequested = TRUE;
DefenseTracker_subzones = {};
DefenseTracker_pname = "";
DefenseTracker_laststamp = 0;

-- Tables for attack summaries
DefenseTracker_attacks = {}; -- [index] = Subzone, Timestamp
DefenseTracker_SubzoneSum = {}; -- [Subzone] = Sum
DefenseTracker_ZoneSum = {}; -- [Zone] = Sum
DefenseTracker_SubzoneTimestamp = {}; -- [Subzone] = timestamp
DefenseTracker_ZoneTimestamp = {}; -- [Zone] = timestamp
DefenseTracker_SortedSubzoneSum = {}; -- [Rank] = Subzone
DefenseTracker_SortedZoneSum = {}; -- [Rank] = Zone
DefenseTracker_SortedSubzoneTimestamp = {}; -- [Rank] = Subzone
DefenseTracker_SortedZoneTimestamp = {}; -- [Rank] = Zone

------------------------------
--      Savedvariables      --
------------------------------
--~ DefenseTracker_newSubzones -- [Subzone] = Zone
--~ DefenseTracker_unknowns -- List of unknown subzones

DefenseTracker_DebugOutput = false;

function DT_DebugFunc(func, args)
	local argstring = "(";
	if args then
		for i=1, table.getn(args) do
			if (not (i==1)) then argstring = argstring.. ", "; end
			argstring = argstring.. args[i];
		end
	end
	argstring = argstring.. ")";
	if (DefenseTracker_DebugOutput) then DEFAULT_CHAT_FRAME:AddMessage("DefTrackDebug: ".. func.. argstring); end
end

function DT_Debug(msg)
	if (DefenseTracker_DebugOutput) then DEFAULT_CHAT_FRAME:AddMessage("DefTrackDebug: ".. msg); end
end

------------------------------
--      Event Handlers      --
------------------------------
function DefenseTracker_OnUpdate(elapsed)
end

function DefenseTracker_OnLoad()
	DT_Debug("DefenseTracker_OnLoad");
	
	this:RegisterEvent("VARIABLES_LOADED");

     if (DEFAULT_CHAT_FRAME) then
         DEFAULT_CHAT_FRAME:AddMessage("|cffffd700DefenseTracker 2.0.4 [|cffBB33BBGoG|cffffd700] Loaded.");
     end

end

function DefenseTracker_ChatFrame_OnEvent(event)
	if ( event == "CHAT_MSG_CHANNEL" ) then
		-- arg1: message
		-- arg9: Channel Name
		if ( arg9 == DEFENSETRACKER_STRINGS_WDCHANNAME ) then
			DT_Debug("Chatter on WD: ".. arg2.. "> ".. arg1);
			local _, _, zone = string.find(arg1, DEFENSETRACKER_STRINGS_ZONEATTACKSEARCH)
			if (zone) then
				DefenseTracker_HandleWorld(zone);
				if (DefenseTracker_options[DefenseTracker_pname]["hideWD"]) then
					DT_Debug("Blocking WD: ".. arg1);
					return false;
				end
			end
		elseif (string.find(arg9, DEFENSETRACKER_STRINGS_LDCHANNAME)) then
			DT_Debug("Chatter on LD: ".. arg2.. "> ".. arg1);
			local _, _, zone = string.find(arg1, DEFENSETRACKER_STRINGS_ZONEATTACKSEARCH)
			if (zone) then
				DefenseTracker_HandleLocal(zone);
				if (DefenseTracker_options[DefenseTracker_pname]["hideLD"]) then
					DT_Debug("Blocking LD: ".. arg1);
					return false;
				end
			end
		end
	end
	
	return DefenseTracker_WoW_ChatFrame_OnEvent(event); 
end


function DefenseTracker_OnEvent(event)
	if (event == "VARIABLES_LOADED") then
		DefenseTracker_Init(); 
	end 
	if ((event == "ZONE_CHANGED_NEW_AREA")
		or (event == "ZONE_CHANGED")
		or (event == "ZONE_CHANGED_INDOORS")) then
		DefenseTracker_handleZoneChange();
	end
end

function DefenseTracker_Init()
	DT_DebugFunc("DefenseTracker_Init");

	this:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	this:RegisterEvent("ZONE_CHANGED");
	this:RegisterEvent("ZONE_CHANGED_INDOORS");

	-- Init some variables
	if (not DefenseTracker_newSubzones) then DefenseTracker_newSubzones = {}; end
	if (not DefenseTracker_unknowns) then DefenseTracker_unknowns = {}; end
	
	DefenseTracker_cleanNewSubzones();
	
	-- Hook the chat event so we can supress server alerts
	DefenseTracker_WoW_ChatFrame_OnEvent = ChatFrame_OnEvent;
	ChatFrame_OnEvent = DefenseTracker_ChatFrame_OnEvent;
	
  DefenseTracker_pname = GetCVar("realmName").. UnitName("player");
  if (not DefenseTracker_options) then DefenseTracker_options = {}; end
	if (not DefenseTracker_options[DefenseTracker_pname]) then
		DefenseTracker_options[DefenseTracker_pname] = {};
		DefenseTracker_options[DefenseTracker_pname]["SZT"] = true;
		DefenseTracker_options[DefenseTracker_pname]["ZT"] = true;
		DefenseTracker_options[DefenseTracker_pname]["SZS"] = true;
		DefenseTracker_options[DefenseTracker_pname]["ZS"] = true;
		DefenseTracker_options[DefenseTracker_pname]["numSZT"] = 5;
		DefenseTracker_options[DefenseTracker_pname]["numZT"] = 5;
		DefenseTracker_options[DefenseTracker_pname]["numSZS"] = 5;
		DefenseTracker_options[DefenseTracker_pname]["numZS"] = 5;
		DefenseTracker_options[DefenseTracker_pname]["hideWD"] = true;
		DefenseTracker_options[DefenseTracker_pname]["hideLD"] = true;
	end

	TitanPanel:AddTimer("TitanPanel"..TITAN_DEFENSE_ID, 5, DefenseTracker_JoinChannels);

end

function DefenseTracker_JoinChannels()
	DT_Debug("DefenseTracker_JoinChannels");
	-- Check that we're in the Def Channels
     DT_Debug("GetChannelNameWD:"..GetChannelName(DEFENSETRACKER_STRINGS_WDCHANNAME));
     DT_Debug("GetChannelNameLD:"..GetChannelName(DEFENSETRACKER_STRINGS_LDCHANNAME));
	if (GetChannelName(DEFENSETRACKER_STRINGS_WDCHANNAME) == 0) then
		JoinChannelByName(DEFENSETRACKER_STRINGS_WDCHANNAME);
		DT_Debug("JoinChannelByName("..DEFENSETRACKER_STRINGS_WDCHANNAME..")");
	end
	if (GetChannelName(DEFENSETRACKER_STRINGS_LDCHANNAME) == 0) then
		JoinChannelByName(DEFENSETRACKER_STRINGS_LDCHANNAME);
		DT_Debug("JoinChannelByName("..DEFENSETRACKER_STRINGS_LDCHANNAME..")");
	end
     -- Remove timer
     local hasTimer = TitanPanel:HasTimer("TitanPanel"..TITAN_DEFENSE_ID)
	if hasTimer then
          TitanPanel:RemoveTimer("TitanPanel"..TITAN_DEFENSE_ID)
     end
end

------------------------------
--      Chat Functions      --
------------------------------
function DefenseTracker_HandleWorld(location)
	if (DefenseTracker_laststamp == time()) then return; end
	DefenseTracker_laststamp = time();
	DT_DebugFunc("DefenseTracker_HandleWorld", {location});
	DefenseTracker_RecordAttack(location, time());
end

function DefenseTracker_HandleLocal(location)
	DT_DebugFunc("DefenseTracker_HandleLocal", {location});
	local zone = GetZoneText();
	DefenseTracker_updateZoneList(location, zone);
end

------------------------------
--Attack Tracking Functions --
------------------------------
function DefenseTracker_RecordAttack(subzone, timestamp)
	DT_DebugFunc("DefenseTracker_RecordAttack", {subzone, timestamp});
	
	table.insert(DefenseTracker_attacks, {subzone, timestamp});
	DefenseTracker_AddToTables(timestamp, subzone);
	DefenseTracker_SortTables();
	TitanPanelButton_UpdateButton(TITAN_DEFENSE_ID);
end

function DefenseTracker_AddToTables(timestamp, subzone)
	local mainzone = DefenseTracker_getMainZone(subzone);

	if (not DefenseTracker_SubzoneSum[subzone]) then DefenseTracker_SubzoneSum[subzone] = 0; end
	if (not DefenseTracker_ZoneSum[mainzone]) then DefenseTracker_ZoneSum[mainzone] = 0; end

	DefenseTracker_SubzoneSum[subzone] = DefenseTracker_SubzoneSum[subzone] + 1;
	DefenseTracker_ZoneSum[mainzone] = DefenseTracker_ZoneSum[mainzone] + 1;
	DefenseTracker_SubzoneTimestamp[subzone] = timestamp;
	DefenseTracker_ZoneTimestamp[mainzone] = timestamp;
end

function DefenseTracker_SortTables()
	DefenseTracker_SortedSubzoneSum = DefenseTracker_Sort(DefenseTracker_SubzoneSum);
	DefenseTracker_SortedZoneSum = DefenseTracker_Sort(DefenseTracker_ZoneSum);
	DefenseTracker_SortedSubzoneTimestamp = DefenseTracker_Sort(DefenseTracker_SubzoneTimestamp);
	DefenseTracker_SortedZoneTimestamp = DefenseTracker_Sort(DefenseTracker_ZoneTimestamp);
end

function DefenseTracker_Sort(intable)
	local table1 = {};	
	local returntable = {};
	for key,val in pairs(intable) do table1[key] = val; end;
	
	local tablesize = 0;
	for key,val in pairs(table1) do tablesize = tablesize+1; end;
	
	for i=1, tablesize do
		local maxval = 0;
		local maxkey = "";
		for key,val in pairs(table1) do
			if (val > maxval) then
				maxval = val;
				maxkey = key;
			end
		end
		returntable[i] = maxkey;
		table1[maxkey] = nil;
	end
	
	return returntable;
end

------------------------------
--  Zone Tracking Functions --
------------------------------
function DefenseTracker_handleZoneChange()
	local subzone = GetSubZoneText();
	local zone = GetZoneText();
	
	if (subzone == nil) then
		subzone = zone;
	end
	if (subzone == "") then
		subzone = zone;
	end

	DefenseTracker_updateZoneList(subzone, zone);
end

function DefenseTracker_updateZoneList(subzone, zone)
	if (DefenseTracker_subzones[subzone] == nil) then
		if (DefenseTracker_newSubzones[subzone] == nil) then
			-- Subzone has not been seen before
			DefenseTracker_newSubzones[subzone] = zone;
			if (DefenseTracker_uploadNotRequested) then
				DEFAULT_CHAT_FRAME:AddMessage(DEFENSETRACKER_STRINGS_NEWZONE1.. subzone.. "/".. zone.. DEFENSETRACKER_STRINGS_NEWZONE2);
				DefenseTracker_uploadNotRequested = FALSE;
			end
			return 1;
		end
	end
	
	return 0;
end

function DefenseTracker_isLocal(subzone)
	local zone = GetZoneText();

	if (DefenseTracker_subzones[subzone] == zone) then
		return true;
	end

	if (DefenseTracker_newSubzones[subzone] == zone) then
		return true;
	end
	
	return false;
end

function DefenseTracker_cleanNewSubzones()
	local DefenseTracker_oldNewSubzones = DefenseTracker_newSubzones;
	DefenseTracker_newSubzones = {};
	
	for loc in pairs(DefenseTracker_oldNewSubzones) do
		if(DefenseTracker_subzones[loc] == nil) then
			DefenseTracker_newSubzones[loc] = DefenseTracker_oldNewSubzones[loc];
		end
	end
end

function DefenseTracker_getMainZone(subzone)
	if (DefenseTracker_subzones[subzone]) then
		return DefenseTracker_subzones[subzone];
	end

	if (DefenseTracker_newSubzones[subzone]) then
		return DefenseTracker_newSubzones[subzone];
	end
	
	if (not DefenseTracker_unknowns[subzone]) then
		DefenseTracker_unknowns[subzone] = 1;
	end
	
	return DEFENSETRACKER_STRINGS_UNKNOWNZONE;

end



------------------------------
--     Output Functions     --
--     Used in plugins      --
------------------------------
function DefenseTracker_GetTimeText(secs)
	local returnstring = "";
	
	if (secs < 60) then
		returnstring = secs.. DEFENSETRACKER_STRINGS_SECONDS;
	elseif (secs < 300) then
		returnstring = math.floor(secs/60*4)/4 .. DEFENSETRACKER_STRINGS_MINUTES;
	elseif (secs < 3600) then
		returnstring = math.floor(secs/60) .. DEFENSETRACKER_STRINGS_MINUTES;
	else
		returnstring = math.floor(secs/3600*4)/4 .. DEFENSETRACKER_STRINGS_HOUR;
	end
	
	return returnstring
end

function DefenseTracker_GetTooltipRawData(dataarray)	
	if (table.getn(DefenseTracker_attacks) == 0) then return nil; end
	
	local returnstring = "";
	
	if dataarray then
		for key,val in pairs(dataarray) do
			returnstring = returnstring.. "[".. key.. "] ".. val.. "\n";
		end
	end
	
	return retrunstring;
end

function DefenseTracker_GetTooltipData(dataarray, numvals, isSubzone)	
	local returnstring = "";
		
	for i=1,numvals do
		if (dataarray[i]) then
			local zone = dataarray[i];
			local atttime = 0;
			local numatt = 0;
			
			if (isSubzone) then
				numatt = DefenseTracker_SubzoneSum[zone];
				atttime = DefenseTracker_SubzoneTimestamp[zone];
			else
				numatt = DefenseTracker_ZoneSum[zone];
				atttime = DefenseTracker_ZoneTimestamp[zone];
			end
			
			local elapsedtime = DefenseTracker_GetTimeText(time() - atttime);
			local mainzone = DefenseTracker_getMainZone(zone);
			
			if (isSubzone) then
				if (DefenseTracker_isLocal(zone)) then
					returnstring = returnstring.. DEFENSETRACKER_STRINGS_LOCALHEADER.. zone.. " (".. numatt.. ")\t".. elapsedtime.. DEFENSETRACKER_STRINGS_AGOENDER.. "\n";
				elseif (mainzone == loc) then
					returnstring = returnstring.. zone.. " (".. numatt.. ")\t".. elapsedtime.. DEFENSETRACKER_STRINGS_AGOENDER.. "\n";
				else
					returnstring = returnstring.. mainzone.. ": ".. zone.. " (".. numatt.. ")\t".. elapsedtime.. DEFENSETRACKER_STRINGS_AGOENDER.. "\n";
				end
			else
				returnstring = returnstring.. zone.. " (".. numatt.. ")\t".. elapsedtime.. DEFENSETRACKER_STRINGS_AGOENDER.. "\n";
			end
		end
	end
	
	return returnstring;
end

function DefenseTracker_getLastAttack()
	if (table.getn(DefenseTracker_attacks) == 0) then return nil; end

	local lasttime = DefenseTracker_attacks[table.getn(DefenseTracker_attacks)][2];
	local lastsubzone = DefenseTracker_attacks[table.getn(DefenseTracker_attacks)][1];
			
	local returnstring = ""..lastsubzone.." ("..DefenseTracker_SubzoneSum[lastsubzone]..")";
	local islocal = false;
	
	if (DefenseTracker_isLocal(lastsubzone)) then
		islocal = true;
	end
	
	return returnstring, islocal, lasttime;
end

function DefenseTracker_GetNumAttText()
	if (table.getn(DefenseTracker_attacks) == 0) then
		return DEFENSETRACKER_STRINGS_NOATTACKS;
	elseif (table.getn(DefenseTracker_attacks) == 1) then
		return DEFENSETRACKER_STRINGS_ATTACKTEXT;
	else
		return table.getn(DefenseTracker_attacks).. DEFENSETRACKER_STRINGS_ATTACKSTEXT;
	end
end

------------------------------
--    Titan Panel Support   --
------------------------------
function TitanPanelDefenseButton_OnLoad()
	this.registry = {
		id = TITAN_DEFENSE_ID,
		menuText = TITAN_DEFENSE_MENU_TEXT,
		buttonTextFunction = "TitanPanelDefenseButton_GetButtonText",
		tooltipTitle = TITAN_DEFENSE_TOOLTIP,
		tooltipTextFunction = "TitanPanelDefenseButton_GetTooltipText",
		frequency = TITAN_DEFENSE_FREQUENCY, 
		icon = "Interface\\Icons\\Ability_Parry.blp";
		iconWidth = 16,
		category = "Information",
		savedVariables = {
			ShowLabelText = 1,
			ShowColoredText = 1,
			ShowIcon = 1,
			SZT = true,
			ZT = true,
			SZS = true,
			ZS = true,
			numSZT = 5,
			numZT = 5,
			numSZS = 5,
			numZS = 5,
			hideWD = true,
			hideLD = true,
		}
	};
end

function TitanPanelDefenseButton_GetButtonText(id)
	local button, id = TitanUtils_GetButton(id, true);
	
	local att, islocal, atttime = DefenseTracker_getLastAttack();
	
	if (not att) then 
		att = DefenseTracker_GetNumAttText();
		return TitanUtils_GetGreenText(att);
	elseif (atttime + TitanDefense_warningDuration < time()) then
		att = DefenseTracker_GetNumAttText();
		return TitanUtils_GetGreenText(att);
	elseif (islocal) then
		return TitanUtils_GetNormalText(att);
	else
		return TitanUtils_GetRedText(att);
	end
end

function TitanPanelDefenseButton_GetTooltipText()
	if (DefenseTracker_numAttacks == 0) then return DEFENSETRACKER_STRINGS_NOATTACKS; end

	local rawdataset = {
		{"------SubzoneSum-----\n", DefenseTracker_SortedSubzoneSum},
		{"------ZoneSum-----\n", DefenseTracker_SortedZoneSum},
		{"------SubzoneTime-----\n", DefenseTracker_SortedSubzoneTimestamp},
		{"------ZoneTime-----\n", DefenseTracker_SortedZoneTimestamp},
		{"------Unknowns-----\n", DefenseTracker_unknowns},
		{"------Attacks-----\n", DefenseTracker_attacks}
	};
	
	local dataset = {
		{DEFENSETRACKER_STRINGS_SZTIMETITLE, DefenseTracker_options[DefenseTracker_pname]["SZT"],
			DefenseTracker_options[DefenseTracker_pname]["numSZT"], DefenseTracker_SortedSubzoneTimestamp, true},
		{DEFENSETRACKER_STRINGS_ZTIMETITLE, DefenseTracker_options[DefenseTracker_pname]["ZT"],
			DefenseTracker_options[DefenseTracker_pname]["numZT"], DefenseTracker_SortedZoneTimestamp, false},
		{DEFENSETRACKER_STRINGS_SZFRQTITLE, DefenseTracker_options[DefenseTracker_pname]["SZS"], 
			DefenseTracker_options[DefenseTracker_pname]["numSZS"], DefenseTracker_SortedSubzoneSum, true},
		{DEFENSETRACKER_STRINGS_ZFRQTITLE, DefenseTracker_options[DefenseTracker_pname]["ZS"], 
			DefenseTracker_options[DefenseTracker_pname]["numZS"], DefenseTracker_SortedZoneSum, false}
	};

	local returnstring = "";
	
	returnstring = returnstring.. DefenseTracker_GetNumAttText().. "\n";
		
--[[
	if (DefenseTracker_DebugOutput) then
		for i,arrayinfo in pairs(rawdataset) do
	     DEFAULT_CHAT_FRAME:AddMessage("DefTrackDebug: ".. arrayinfo[1]..":".. arrayinfo[2]);
			if (arrayinfo[2]) then
				returnstring = returnstring..arrayinfo[1];
				returnstring = returnstring..DefenseTracker_GetTooltipRawData(arrayinfo[2]);
				returnstring = returnstring.."\n";
			end
		end
	end
]]

	for i,arrayinfo in pairs(dataset) do
		if (arrayinfo[2]) then
			returnstring = returnstring.."\n".. TitanUtils_GetHighlightText(arrayinfo[1]).. "\n";
			returnstring = returnstring..DefenseTracker_GetTooltipData(arrayinfo[4], arrayinfo[3], arrayinfo[5]);
		end
	end
	
	return returnstring;
end

function TitanPanelRightClickMenu_PrepareDefenseMenu()
	if (not DefenseTracker_options) or (not DefenseTracker_options[DefenseTracker_pname]) then return false; end
	
	local id = TITAN_DEFENSE_ID;
	local info = {};

	local items = {
		{"Supress WorldDef Alerts", "hideWD"},
		{"Supress LocalDef Alerts", "hideLD"},
		{"Show Recent Subzones", "SZT"},
		{"Show Recent Zones", "ZT"},
		{"Show Freq Subzones", "SZS"},
		{"Show Freq Zones", "ZS"},
	};
	
	local items2 = {
		{"numSZT", 5},
		{"numZT", 5},
		{"numSZS", 5},
		{"numZS", 5},
	};
	
	for i,arr in pairs(items) do
		info.text = arr[1];
		info.value = arr[2];
		info.checked = DefenseTracker_options[DefenseTracker_pname][arr[2]];
		info.func = TitanPanelDefense_Toggle;
		UIDropDownMenu_AddButton(info);
	end
	
	info = {};
	info.disabled = 1;
	UIDropDownMenu_AddButton(info);
		
	TitanPanelRightClickMenu_AddCommand(TITAN_PANEL_MENU_CUSTOMIZE..TITAN_PANEL_MENU_POPUP_IND, id, TITAN_PANEL_MENU_FUNC_CUSTOMIZE);
	TitanPanelRightClickMenu_AddCommand(TITAN_PANEL_MENU_HIDE, id, TITAN_PANEL_MENU_FUNC_HIDE);
end

function TitanPanelDefense_Toggle()
	local setting = this.value;

	if (DefenseTracker_options[DefenseTracker_pname][setting]) then 
		DefenseTracker_options[DefenseTracker_pname][setting] = false;
	else 
		DefenseTracker_options[DefenseTracker_pname][setting] = true;
	end
end